@@ -25,6 +25,7 @@
 #include "adium-theme-message-info.h"
 #include "adium-theme-status-info.h"
 #include "channel-contact-model.h"
+#include "logmanager.h"
 
 #include <QtGui/QKeyEvent>
 #include <QtGui/QAction>
@@ -44,52 +45,6 @@
 #include <TelepathyQt4/Connection>
 #include <TelepathyQt4/Presence>
 
-
-class MessageBoxEventFilter : public QObject
-{
-    Q_OBJECT
-public:
-    MessageBoxEventFilter(QObject* parent = 0) : QObject(parent) {}
-
-protected:
-    virtual bool eventFilter(QObject *obj, QEvent *event)
-    {
-        if (event->type() == QEvent::KeyPress) {
-            QKeyEvent *keyEvent = static_cast<QKeyEvent *>(event);
-            if (keyEvent->key() == Qt::Key_Enter || keyEvent->key() == Qt::Key_Return) {
-                if (!keyEvent->modifiers()) {
-                    Q_EMIT returnKeyPressed();
-                    return true;
-                }
-            }
-        }
-        // standard event processing
-        return QObject::eventFilter(obj, event);
-    }
-
-Q_SIGNALS:
-    void returnKeyPressed();
-};
-
-class WindowEventFilter : public QObject
-{
-    Q_OBJECT
-public:
-    WindowEventFilter(QObject* parent = 0) : QObject(parent) {}
-
-protected:
-    virtual bool eventFilter(QObject *obj, QEvent *event)
-    {
-        if (event->type() == QEvent::ActivationChange) {
-            Q_EMIT windowActivated();
-        }
-        return QObject::eventFilter(obj, event);
-    }
-
-Q_SIGNALS:
-    void windowActivated();
-};
-
 class ChatWidgetPrivate
 {
 public:
@@ -110,6 +65,7 @@ public:
     Tp::AccountPtr account;
     Ui::ChatWidget ui;
     ChannelContactModel *contactModel;
+    LogManager *logManager;
 
     KComponentData telepathyComponentData();
 };
@@ -129,6 +85,9 @@ ChatWidget::ChatWidget(const Tp::TextChannelPtr & channel, const Tp::AccountPtr
     d->channel = channel;
     d->account = account;
 
+    //load translations for this library. keep this before any i18n() calls in library code
+    KGlobal::locale()->insertCatalog(QLatin1String("ktelepathy_chat_lib"));
+
     d->chatviewlInitialised = false;
     d->showFormatToolbarAction = new QAction(i18n("Show format options"), this);
     d->isGroupChat = (channel->targetHandleType() == Tp::HandleTypeContact ? false : true);
@@ -136,19 +95,19 @@ ChatWidget::ChatWidget(const Tp::TextChannelPtr & channel, const Tp::AccountPtr
     d->ui.setupUi(this);
     d->ui.formatToolbar->show();
     d->ui.formatColor->setText(QString());
-    d->ui.formatColor->setIcon(KIcon("format-text-color"));
+    d->ui.formatColor->setIcon(KIcon(QLatin1String("format-text-color")));
 
     d->ui.formatBold->setText(QString());
-    d->ui.formatBold->setIcon(KIcon("format-text-bold"));
+    d->ui.formatBold->setIcon(KIcon(QLatin1String("format-text-bold")));
 
     d->ui.formatItalic->setText(QString());
-    d->ui.formatItalic->setIcon(KIcon("format-text-italic"));
+    d->ui.formatItalic->setIcon(KIcon(QLatin1String("format-text-italic")));
 
     d->ui.formatUnderline->setText(QString());
-    d->ui.formatUnderline->setIcon(KIcon("format-text-underline"));
+    d->ui.formatUnderline->setIcon(KIcon(QLatin1String("format-text-underline")));
 
     d->ui.insertEmoticon->setText(QString());
-    d->ui.insertEmoticon->setIcon(KIcon("face-smile"));
+    d->ui.insertEmoticon->setIcon(KIcon(QLatin1String("face-smile")));
 
     // connect channel signals
     setupChannelSignals();
@@ -159,6 +118,8 @@ ChatWidget::ChatWidget(const Tp::TextChannelPtr & channel, const Tp::AccountPtr
 
     d->ui.contactsView->setModel(d->contactModel);
 
+    d->ui.chatArea->load((d->isGroupChat?AdiumThemeView::GroupChat:AdiumThemeView::SingleUserChat));
+
     AdiumThemeHeaderInfo info;
 
     //normal chat - self and one other person.
@@ -209,17 +170,9 @@ ChatWidget::ChatWidget(const Tp::TextChannelPtr & channel, const Tp::AccountPtr
     // make the sendMessageBox a focus proxy for the chatview
     d->ui.chatArea->setFocusProxy(d->ui.sendMessageBox);
 
-    MessageBoxEventFilter *messageBoxEventFilter = new MessageBoxEventFilter(this);
-    d->ui.sendMessageBox->installEventFilter(messageBoxEventFilter);
-    connect(messageBoxEventFilter, SIGNAL(returnKeyPressed()), SLOT(sendMessage()));
+    connect(d->ui.sendMessageBox, SIGNAL(returnKeyPressed()), SLOT(sendMessage()));
     connect(d->ui.sendButton, SIGNAL(clicked()), SLOT(sendMessage()));
 
-    WindowEventFilter *windowEventFilter = new WindowEventFilter(this);
-    this->window()->installEventFilter(windowEventFilter);
-    connect(windowEventFilter, SIGNAL(windowActivated()), SLOT(windowActivated()));
-
-    // find text in chat
-    connect(d->ui.sendMessageBox, SIGNAL(findTextShortcutPressed()), this, SLOT(toggleSearchBar()));
     connect(d->ui.searchBar, SIGNAL(findTextSignal(QString,QWebPage::FindFlags)), this, SLOT(findTextInChat(QString,QWebPage::FindFlags)));
     connect(d->ui.searchBar, SIGNAL(findNextSignal(QString,QWebPage::FindFlags)), this, SLOT(findNextTextInChat(QString,QWebPage::FindFlags)));
     connect(d->ui.searchBar, SIGNAL(findPreviousSignal(QString,QWebPage::FindFlags)), this, SLOT(findPreviousTextInChat(QString,QWebPage::FindFlags)));
@@ -227,13 +180,16 @@ ChatWidget::ChatWidget(const Tp::TextChannelPtr & channel, const Tp::AccountPtr
 
     connect(this, SIGNAL(searchTextComplete(bool)), d->ui.searchBar, SLOT(onSearchTextComplete(bool)));
 
-	// to make PgUp and PgDown keys work properly
-	connect(d->ui.sendMessageBox, SIGNAL(scrollEventRecieved(QKeyEvent*)), d->ui.chatArea, SLOT(onScrollEvent(QKeyEvent*)));
+    // initialize LogManager
+    d->logManager = new LogManager(account, channel->targetContact(), this);
+    d->logManager->setFetchAmount(3);
+    d->logManager->setTextChannel(channel);
 }
 
 ChatWidget::~ChatWidget()
 {
     d->channel->requestClose(); // ensure closing; does nothing, if already closed
+    delete d->logManager;
     delete d;
 }
 
@@ -271,7 +227,7 @@ KIcon ChatWidget::icon() const
         //normal chat - self and one other person.
         if (!d->isGroupChat) {
             //find the other contact which isn't self.
-            foreach(const Tp::ContactPtr & contact, d->channel->groupContacts()) {
+            Q_FOREACH(const Tp::ContactPtr & contact, d->channel->groupContacts()) {
                 if (contact != d->channel->groupSelfContact()) {
                     return iconForPresence(contact->presence().type());
                 }
@@ -329,22 +285,25 @@ Tp::TextChannelPtr ChatWidget::textChannel() const
     return d->channel;
 }
 
-void ChatWidget::showEvent(QShowEvent* e)
-{
-    kDebug();
-
-    resetUnreadMessageCount();
-
-    QWidget::showEvent(e);
-}
-
 void ChatWidget::keyPressEvent(QKeyEvent* e)
 {
+    if (e->matches(QKeySequence::Copy)) {
+        d->ui.chatArea->triggerPageAction(QWebPage::Copy);
+        return;
+    }
+
     if (e->key() == Qt::Key_Escape && d->ui.searchBar->isVisible()) {
         d->ui.searchBar->toggleView(false);
-    } else {
-        QWidget::keyPressEvent(e);
+        return;
     }
+
+    if (e->key() == Qt::Key_PageUp ||
+        e->key() == Qt::Key_PageDown) {
+        d->ui.chatArea->event(e);
+        return;
+    }
+
+    QWidget::keyPressEvent(e);
 }
 
 QString ChatWidget::title() const
@@ -377,7 +336,7 @@ QColor ChatWidget::titleColor() const
     //normal chat - self and one other person.
     if (!d->isGroupChat) {
         //find the other contact which isn't self.
-        foreach(const Tp::ContactPtr & contact, d->channel->groupContacts()) {
+        Q_FOREACH(const Tp::ContactPtr & contact, d->channel->groupContacts()) {
             if (contact != d->channel->groupSelfContact()) {
                 if (contact->presence().type() == Tp::ConnectionPresenceTypeOffline ||
                     contact->presence().type() == Tp::ConnectionPresenceTypeHidden) {
@@ -442,14 +401,25 @@ void ChatWidget::incrementUnreadMessageCount()
     Q_EMIT unreadMessagesChanged(d->unreadMessages);
 }
 
-void ChatWidget::windowActivated()
+void ChatWidget::onHistoryFetched(const QList<AdiumThemeContentInfo> &messages)
 {
-    kDebug();
-    if (isOnTop()) {
-        resetUnreadMessageCount();
+    kDebug() << "found" << messages.count() << "messages in history";
+    Q_FOREACH(const AdiumThemeContentInfo &message, messages) {
+        d->ui.chatArea->addContentMessage(message);
     }
+
+    d->chatviewlInitialised = true;
+
+    //process any messages we've 'missed' whilst initialising.
+    Q_FOREACH(const Tp::ReceivedMessage &message, d->channel->messageQueue()) {
+        handleIncomingMessage(message);
+    }
+
+    delete d->logManager;
+    d->logManager = 0;
 }
 
+
 void ChatWidget::resetUnreadMessageCount()
 {
     kDebug();
@@ -470,61 +440,104 @@ void ChatWidget::handleIncomingMessage(const Tp::ReceivedMessage &message)
 {
     kDebug() << title() << message.text();
 
-    if (message.isDeliveryReport()) {
-        ///TODO what to do with this info? For now just output to cli
-        // we don't need to show this in chat
-        Tp::ReceivedMessage::DeliveryDetails reportDetails = message.deliveryDetails();
+    if (d->chatviewlInitialised) {
 
-        switch (reportDetails.status()) {
-            case (Tp::DeliveryStatusAccepted):
-                kDebug() << "ChatWidget::handleIncomingMessage DeliveryStatusAccepted";
-                break;
-            case (Tp::DeliveryStatusDeleted):
-                kDebug() << "ChatWidget::handleIncomingMessage DeliveryStatusDeleted";
-                break;
-            case (Tp::DeliveryStatusDelivered):
-                kDebug() << "ChatWidget::handleIncomingMessage DeliveryStatusDelivered";
-                break;
-            case (Tp::DeliveryStatusPermanentlyFailed):
-                kDebug() << "ChatWidget::handleIncomingMessage DeliveryStatusPermanentlyFailed";
-                break;
-            case (Tp::DeliveryStatusRead):
-                kDebug() << "ChatWidget::handleIncomingMessage DeliveryStatusRead";
-                break;
-            case (Tp::DeliveryStatusTemporarilyFailed):
-                kDebug() << "ChatWidget::handleIncomingMessage DeliveryStatusTemporarilyFailed";
-                break;
-            case (Tp::DeliveryStatusUnknown):
-                kDebug() << "ChatWidget::handleIncomingMessage DeliveryStatusUnknown";
-                break;
-        }
+        //debug the message parts (looking for HTML etc)
+//        Q_FOREACH(Tp::MessagePart part, message.parts())
+//        {
+//            qDebug() << "***";
+//            Q_FOREACH(QString key, part.keys())
+//            {
+//                qDebug() << key << part.value(key).variant();
+//            }
+//        }
+//      turns out we have no HTML, because no CM supports it yet
 
-        if (reportDetails.isError()) {
-            kDebug() << "ChatWidget::handleIncomingMessage ERROR: " << reportDetails.error();
-            kDebug() << "ChatWidget::handleIncomingMessage DBUS ERROR: " << reportDetails.dbusError();
-        }
+        if (message.isDeliveryReport()) {
+            QString text;
+            AdiumThemeStatusInfo messageInfo;
+            Tp::ReceivedMessage::DeliveryDetails reportDetails = message.deliveryDetails();
 
-        if (reportDetails.hasDebugMessage()) {
-            kDebug() << "ChatWidget::handleIncomingMessage DEBUG MESSAGE: " << reportDetails.debugMessage();
-        }
-    } else {
-        if (d->chatviewlInitialised) {
+            if (reportDetails.hasDebugMessage()) {
+                kDebug() << "delivery report debug message: " << reportDetails.debugMessage();
+            }
+
+            if (reportDetails.isError()) {
+                switch (reportDetails.error()) {
+                case Tp::ChannelTextSendErrorOffline:
+                    if (reportDetails.hasEchoedMessage()) {
+                        text = i18n("Delivery of the message \"%1\" "
+                                    "failed because the remote contact is offline",
+                                    reportDetails.echoedMessage().text());
+                    } else {
+                        text = i18n("Delivery of a message failed "
+                                    "because the remote contact is offline");
+                    }
+                    break;
+                case Tp::ChannelTextSendErrorInvalidContact:
+                    if (reportDetails.hasEchoedMessage()) {
+                        text = i18n("Delivery of the message \"%1\" "
+                                    "failed because the remote contact is not valid",
+                                    reportDetails.echoedMessage().text());
+                    } else {
+                        text = i18n("Delivery of a message failed "
+                                    "because the remote contact is not valid");
+                    }
+                    break;
+                case Tp::ChannelTextSendErrorPermissionDenied:
+                    if (reportDetails.hasEchoedMessage()) {
+                        text = i18n("Delivery of the message \"%1\" failed because "
+                                    "you do not have permission to speak in this room",
+                                    reportDetails.echoedMessage().text());
+                    } else {
+                        text = i18n("Delivery of a message failed because "
+                                    "you do not have permission to speak in this room");
+                    }
+                    break;
+                case Tp::ChannelTextSendErrorTooLong:
+                    if (reportDetails.hasEchoedMessage()) {
+                        text = i18n("Delivery of the message \"%1\" "
+                                    "failed because it was too long",
+                                    reportDetails.echoedMessage().text());
+                    } else {
+                        text = i18n("Delivery of a message failed "
+                                    "because it was too long");
+                    }
+                    break;
+                default:
+                    if (reportDetails.hasEchoedMessage()) {
+                        text = i18n("Delivery of the message \"%1\" failed: %2",
+                                    reportDetails.echoedMessage().text(),
+                                    message.text());
+                    } else {
+                        text = i18n("Delivery of a message failed: %1", message.text());
+                    }
+                    break;
+                }
+            } else {
+                //TODO: handle delivery reports properly
+                kWarning() << "Ignoring delivery report";
+                d->channel->acknowledge(QList<Tp::ReceivedMessage>() << message);
+                return;
+            }
+
+            messageInfo.setMessage(text);
+            messageInfo.setTime(message.received());
+            messageInfo.setStatus(QLatin1String("error"));
+
+            d->ui.chatArea->addStatusMessage(messageInfo);
+        } else {
             AdiumThemeContentInfo messageInfo(AdiumThemeMessageInfo::RemoteToLocal);
 
-            //debug the message parts (looking for HTML etc)
-    //        foreach(Tp::MessagePart part, message.parts())
-    //        {
-    //            qDebug() << "***";
-    //            foreach(QString key, part.keys())
-    //            {
-    //                qDebug() << key << part.value(key).variant();
-    //            }
-    //        }
-    //      turns out we have no HTML, because no CM supports it yet
             messageInfo.setMessage(message.text());
-            messageInfo.setTime(message.received());
 
-            if (message.sender() == 0) {
+            QDateTime time = message.sent();
+            if (!time.isValid()) {
+                time = message.received();
+            }
+            messageInfo.setTime(time);
+
+            if (message.sender().isNull()) {
                 // just need this info
                 messageInfo.setSenderDisplayName(message.senderNickname());
             } else {
@@ -534,17 +547,18 @@ void ChatWidget::handleIncomingMessage(const Tp::ReceivedMessage &message)
             }
 
             d->ui.chatArea->addContentMessage(messageInfo);
-            d->channel->acknowledge(QList<Tp::ReceivedMessage>() << message);
-
-            if (!isOnTop()) {
-                incrementUnreadMessageCount();
-            }
-
-            Q_EMIT messageReceived();
         }
+
+        d->channel->acknowledge(QList<Tp::ReceivedMessage>() << message);
+
+        if (!isOnTop()) {
+            incrementUnreadMessageCount();
+        }
+
+        Q_EMIT messageReceived();
     }
 
-    //if the window isn't ready, we don't acknowledge the mesage. We process them as soon as we are ready.
+    //if the window isn't ready, we don't acknowledge the message. We process them as soon as we are ready.
 }
 
 void ChatWidget::notifyAboutIncomingMessage(const Tp::ReceivedMessage & message)
@@ -587,9 +601,15 @@ void ChatWidget::notifyAboutIncomingMessage(const Tp::ReceivedMessage & message)
         notification->setPixmap(notificationPixmap);
     }
 
-    notification->setText(message.text());
+    if (message.text().length() > 170) {
+        //search for the closest space in text
+        QString truncatedMsg = message.text().left(message.text().indexOf(QLatin1Char(' '), 150)).append(QLatin1String("..."));
+        notification->setText(truncatedMsg);
+    } else {
+        notification->setText(message.text());
+    }
     //allows per contact notifications
-    notification->addContext("contact", message.sender()->id());
+    notification->addContext(QLatin1String("contact"), message.sender()->id());
     //TODO notification->addContext("group",... Requires KDE Telepathy Contact to work out which group they are in.
 
     notification->setActions(QStringList(i18n("View")));
@@ -625,12 +645,8 @@ void ChatWidget::handleMessageSent(const Tp::Message &message, Tp::MessageSendin
 
 void ChatWidget::chatViewReady()
 {
-    d->chatviewlInitialised = true;
-
-    //process any messages we've 'missed' whilst initialising.
-    foreach(Tp::ReceivedMessage message, d->channel->messageQueue()) {
-        handleIncomingMessage(message);
-    }
+    connect(d->logManager, SIGNAL(fetched(QList<AdiumThemeContentInfo>)), SLOT(onHistoryFetched(QList<AdiumThemeContentInfo>)));
+    d->logManager->fetchLast();
 }
 
 
@@ -657,7 +673,7 @@ void ChatWidget::onChatStatusChanged(const Tp::ContactPtr & contact, Tp::Channel
         AdiumThemeStatusInfo statusMessage;
         statusMessage.setMessage(i18n("%1 has left the chat", contact->alias()));
         statusMessage.setService(d->channel->connection()->protocolName());
-        statusMessage.setStatus("away");
+        statusMessage.setStatus(QLatin1String("away"));
         statusMessage.setTime(QDateTime::currentDateTime());
         d->ui.chatArea->addStatusMessage(statusMessage);
         break;
@@ -672,13 +688,13 @@ void ChatWidget::onChatStatusChanged(const Tp::ContactPtr & contact, Tp::Channel
         contactIsTyping = true;
         break;
     default:
-        qDebug() << QString("Unknown case %1").arg(state);
+        kWarning() << "Unknown channel chat case" << state;
     }
 
     if (!contactIsTyping) {
         //In a multiperson chat just because this user is no longer typing it doesn't mean that no-one is.
         //loop through each contact, check no-one is in composing mode.
-        foreach (const Tp::ContactPtr & contact, d->channel->groupContacts()) {
+        Q_FOREACH (const Tp::ContactPtr & contact, d->channel->groupContacts()) {
             if (contact == d->channel->connection()->selfContact()) {
                 continue;
             }
@@ -806,9 +822,9 @@ void ChatWidget::findTextInChat(const QString& text, QWebPage::FindFlags flags)
     d->ui.chatArea->findText(QString(), flags);
 
     if(d->ui.chatArea->findText(text, flags)) {
-        emit(searchTextComplete(true));
+        Q_EMIT searchTextComplete(true);
     } else {
-        emit(searchTextComplete(false));
+        Q_EMIT searchTextComplete(false);
     }
 }
 
@@ -859,5 +875,9 @@ KIcon ChatWidget::iconForPresence(Tp::ConnectionPresenceType presence)
     return KIcon(iconName);
 }
 
-#include "chat-widget.moc" //for MessageBoxEventFilter
-#include "moc_chat-widget.cpp" //for ChatWidget
+bool ChatWidget::isUserTyping() const
+{
+    return d->remoteContactIsTyping;
+}
+
+#include "chat-widget.moc"
